<?php
/**
 * This file is part of the Agora distribution. 
 * Detailed copyright and licensing information can be found
 * in the gpl-3.0.txt file which should be included in the distribution.
 * 
 * @version		$Id: 3.0.14 Olympus 2010-09-28 rammstein4o $
 * @copyright  2007 - 2010 jVitals
 * @license   GPLv3 Open Source
 * @link       http://jvitals.com
 * @since      File available since initial release
 */
defined ('IN_AGORA') or die;

class TaskController extends AgoraPollsController
{
    function _default()
    {
        $catnav_helper = & $this->helper('catnav');
        $catnav_helper->fromForum($this->id);

		$access_helper = & $this->helper('access');
		$access = $access_helper->getForumAccess($this->id);
		$this->view->assign('access',$access);

        $this->view->template = 'poll/multi_yes_no';
    }

    function create()
    {
		if (!$this->agora_user['is_superadmin'] &&
			$this->access_model->authenticate($this->agora_user['id'],$this->id,'use_captcha')) {
			$captcha = Agora::getPostVar('captcha');
			$c_key = Agora::getSessionVar('captcha');

			if (trim($captcha) !== trim($c_key) || !$captcha) {
				Agora::showMessage(Agora::lang('Bad captcha'));
				Agora::redirect(Agora::getRefferer());
			}
		}
		
        $poll_options = Agora::getPostVar('poll_option',array());
        $opt = array();
        foreach ($poll_options as $option) {
            $option = trim($option);
            if (!empty($option)) {
                $opt[] = $option;
            }
        }

        if (count($opt) < 2) {
            Agora::showMessage(Agora::lang('You need to set at least 2 options'));
            Agora::redirect(Agora::getRefferer());
        }

        if (trim(Agora::getPostVar('subject',''))=='') {
            Agora::showMessage(Agora::lang('You need to set subject'));
            Agora::redirect(Agora::getRefferer());
        }

        $poll_yes = Agora::getPostVar('poll_yes','yes');
        $poll_no = Agora::getPostVar('poll_no','no');

        $poll_yes = trim($poll_yes);
        $poll_no = trim($poll_no);

        if (!$poll_yes || !$poll_no) {
            Agora::showMessage(Agora::lang('You need to set values for both YES and NO fields'));
            Agora::redirect(Agora::getRefferer());
        }

        $topic_model = & Model::getInstance('TopicModel');
				
		JPluginHelper::importPlugin( 'agora' );
		$dispatcher =& JDispatcher::getInstance();
		$options= new JObject; 
		
		$options->forum_id 	= $this->id ;
		$options->topic_id 	= NULL  ;
		$options->subject 	= Agora::getPostVar('subject','No subject') ;
		$options->message 	= Agora::getPostVar('req_message','',true) ;
		$options->desc 		= Agora::getPostVar('desc','') ;
		$options->user_name = $this->agora_user['username'] ;
		$options->user_id 	= $this->agora_user['id'] ;
		$options->question 	= Agora::getPostVar('req_question','no question');
		$options->poll_yes 	= $poll_yes  ;
		$options->poll_no 	= $poll_no  ;
		$subscribe = Agora::getPostVar('subscribe',0);
		
		$plg_arguments = array(&$options);
		
		$plg_result =  $dispatcher->trigger( 'onBeforeCreatePoll',$plg_arguments) ;

		// addPoll($forum_id, $subject,$desc,$icon, $user_name, $question, $yes = NULL, $no = NULL)
        $topic_id = $topic_model->addPoll(
            $this->id,
            Agora::getPostVar('subject','No subject'),
            Agora::getPostVar('desc',''),
            intval(Agora::getPostVar('topic_icon',0)),
            $this->agora_user['username'],
            Agora::getPostVar('req_question','no question'),
            $poll_yes,
            $poll_no
        );
		// addPost($topic_id, $forum_id, $message, $user_id, $username, $hide_smilies)
        $topic_model->addPost(
            $topic_id,
            $this->id,
            Agora::getPostVar('req_message','',true),
            $this->agora_user['id'],
            $this->agora_user['username'],
            Agora::getPostVar('hide_smilies',0)
        );

        $this->model->add($topic_id, $opt,	3);
		
		$options->topic_id 	= $topic_id  ;
		$plg_arguments = array(&$options);
		
    		if ( $subscribe ) {
			$subscription_model = & Model::getInstance('SubscriptionModel');
			$user_id = $this->agora_user['id'];

			$subscription_model->add(array('user_id'=>$user_id,'topic_id'=>$topic_id));
			Agora::showMessage(Agora::lang('You are now subscribed on topic ').AString::escape($options->subject)); 
		}
		
		$plg_result =  $dispatcher->trigger( 'onAfterCreatePoll',$plg_arguments) ;
        Agora::redirect(Agora::makeUrl(array('task'=>'topic','id'=>$topic_id)));
    }

    function vote()
    {
        $poll = $this->model->load($this->id,'pollid');

        if (!$poll) {
            Agora::showError(Agora::lang('Invalid request (bad poll id)'));
            $this->redirect();
        }
        if ($poll['ptype'] != '3') {
            Agora::showError(Agora::lang('Invalid request (bad poll type)'));
            $this->redirect();
        }

        $voters = $poll['voters'];
        $votes = $poll['votes'];
        $voter = intval($this->agora_user['id']);

        if (!is_array($votes)) $votes = array();
        if (!is_array($voters)) $voters = array();

        if (in_array($voter,$voters)) {
            Agora::showError(Agora::lang('You already voted'));
            $this->redirect();
        }

        if (Agora::getPostVar('submit')) {
            $options = Agora::getPostVar('options',array());

            foreach ($options as $key=>$value) {

                if ($value == "yes")
                {
                    if (isset($votes[$key]['yes']))
                    $votes[$key]['yes']++;
                    else
                    $votes[$key]['yes'] = 1;
                } else {
                    if (isset($votes[$key]['no']))
                    $votes[$key]['no']++;
                    else
                    $votes[$key]['no'] = 1;
                }
            }
        }

        $voters[] = $voter;

        $this->model->setVotes($this->id, $votes,$voters);

        Agora::showMessage(Agora::lang('You vote accepted'));
        $this->redirect();
    }
}
?>
